package com.softslate.commerce.businessobjects.customer;

import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Random;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.ApplicationException;
import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.businessobjects.core.DateUtils;
import com.softslate.commerce.businessobjects.order.CartDiscountProcessor;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderProcessor;
import com.softslate.commerce.businessobjects.product.ProductSetting;
import com.softslate.commerce.daos.customer.CustomerDAO;
import com.softslate.commerce.daos.customer.CustomerGatewayDAO;
import com.softslate.commerce.daos.order.OrderDAO;

/**
 * Processes business logic having to do with a customer's account and related
 * objects. Created in the Struts layer in response to requests for information
 * about an exisiting customer account, or to create new accounts.
 * <p>
 * <code>BasicCustomerProcessor</code> is the default implementation of the
 * <code>CustomerProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * @see com.softslate.commerce.businessobjects.customer.CustomerProcessor
 */
public class BasicCustomerProcessor extends BaseBusinessProcessor implements
		CustomerProcessor {
	static Log log = LogFactory.getLog(BasicCustomerProcessor.class);

	public Map processRegister(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing customer registration.");
		Map result = new HashMap();
		if (getUser().getCustomer() != null) {
			result.put("resultCode", "errors.userAlreadyLoggedIn");
			return result;
		}
		// Initialize the customer object
		getUser().setCustomer(
				(Customer) getBusinessObjectFactory().createObject(
						"customerImplementer"));
		if (parameters != null) {
			BeanUtils.copyProperties(getUser().getCustomer(), parameters);
		}
		getUser().getCustomer().setStatus("Registered");
		getUser().getCustomer().setCreated(formatDateTime(new Date()));
		getUser().getCustomer().setLastLogin(formatDateTime(new Date()));
		CustomerGatewayDAO customerGatewayDAO =  getDaoFactory()
				.createDAO(CustomerGatewayDAO.class);
		result = customerGatewayDAO.processRegister(getUser());
		// Set the customer to null and return if creating the account failed.
		if (result.get("resultCode") != null
				&& !result.get("resultCode").equals("0")) {
			getUser().setCustomer(null);
			return result;
		}
		// The account was created successfully.
		// If the user has an active order, copy in the customer and
		// userName, and rerun discounts
		if (getUser().getOrder() != null) {
			getUser().getOrder().setCustomer(getUser().getCustomer());
			getUser().getOrder().setUserName(
					getUser().getCustomer().getUserName());
			CartDiscountProcessor discountProcessor =  getBusinessObjectFactory()
					.createObject(CartDiscountProcessor.class);
			Map newResults = discountProcessor.processCustomerLogin(null);
			if (newResults != null) {
				result.putAll(newResults);
			}
		}
		result.put("resultCode", "0");
		return result;
	}

	public Map processLogin(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing customer login.");
		Map result = new HashMap();
		if (getUser().getCustomer() != null) {
			if (log.isInfoEnabled())
				log.info("Customer is already logged in.");
			result.put("resultCode", "errors.userAlreadyLoggedIn");
			return result;
		}
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO("customerDAOImplementer");

		// Load some settings and encrypt the password
		int numberOfPreviousPasswordsToCheck = 0;
		String numberOfPreviousPasswordsToCheckStr = getSettings().getValue("numberOfPreviousPasswordsToCheckCustomer");
		if (numberOfPreviousPasswordsToCheckStr != null) {
			numberOfPreviousPasswordsToCheck = Integer.parseInt(numberOfPreviousPasswordsToCheckStr);
		}
		String newDecryptedPassword = (String) parameters.get("newDecryptedPassword");
		String newPasswordEncrypted = null;
		if (newDecryptedPassword != null) {
			newPasswordEncrypted = customerDAO.getDAOUtils().getEncrypted(newDecryptedPassword,
					getSettings().getValue("customerEncryptionType"),
					getAppSettings().getProperty("twoWayKeyFile"),
					getSettings().getValue("oneWayEncryptionAlgorithm"));
		}

		// Prep to load the customer
		Customer tempCustomer = (Customer) getBusinessObjectFactory().createObject("customerImplementer");
		if (parameters != null) {
			BeanUtils.copyProperties(tempCustomer, parameters);
		}
		customerDAO.setCustomer(tempCustomer);

		// Save the autoLoginToken, if there is one
		String autoLoginToken = tempCustomer.getAutoLoginToken();

		// Load the customer
		customerDAO.loadFromUserNameAndPassword();
		Customer validCustomer = customerDAO.getCustomer();
		if (validCustomer == null) {

			// Login failed. Try to load the customer by just the userName, and add one to the failed login count, and
			// if too many, lock the customer out. If the lock out period has expired though, reset the count so they
			// get a new set of attempts.
			getUser().setCustomer(null);
			customerDAO.setCustomer(tempCustomer);
			customerDAO.loadFromUserName();
			tempCustomer = customerDAO.getCustomer();
			if (tempCustomer != null) {
				resetNumberOfFailedLoginsIfLockOutExpired(tempCustomer);
				incrementNumberOfFailedLogins(tempCustomer);
			}
			if (tempCustomer != null && !isCustomerLockedOut(tempCustomer)) {
				lockOutIfTooManyFailedLogins(tempCustomer);
			}

			// Whether the login was valid or not, show the locked out error if the user is locked out.
			if (isCustomerLockedOut(tempCustomer)) {
				result.put("resultCode", "errors.customerLockedOut");
			} else {
				result.put("resultCode", "errors.invalidLogin");
			}
		} else if (isCustomerLockedOut(validCustomer)) {
			result.put("resultCode", "errors.customerLockedOut");
			result.put("validCustomer", null);
			getUser().setCustomer(null);
		} else if (newDecryptedPassword == null && isPasswordExpired(validCustomer)) {
			result.put("resultCode", "errors.passwordHasExpired");
			result.put("validCustomer", null);
			getUser().setCustomer(null);
		} else if (newDecryptedPassword != null
				&& isSameAsPreviousPassword(validCustomer, newPasswordEncrypted, numberOfPreviousPasswordsToCheck)) {
			result.put("resultCode", "errors.sameAsPreviousPassword");
			result.put("numberOfPreviousPasswordsToCheck", numberOfPreviousPasswordsToCheck);
			result.put("validCustomer", null);
			getUser().setCustomer(null);
		} else {

			validCustomer.setLastLogin(formatDateTime(new Date()));
			validCustomer.setAutoLoginToken(autoLoginToken);
			validCustomer.setNumberOfFailedLogins(0);
			validCustomer.setLockedOutUntil(null);
			if (newDecryptedPassword != null) {
				customerChangePassword(newDecryptedPassword, validCustomer);
			}
			getUser().setCustomer(validCustomer);

			// Load the customer addresses and wish lists
			Collection addresses = customerDAO.loadAddressCollection();

			// Load the customer wish lists
			Collection wishLists = customerDAO.loadCustomerWishLists();

			// Add the customer's ID and user name to the order, if one exists.
			if (getUser().getOrder() != null) {
				OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO("orderDAOImplementer");
				orderDAO.setOrder(getUser().getOrder());
				getUser().getOrder().setCustomer(validCustomer);
				getUser().getOrder().setUserName(validCustomer.getUserName());
				orderDAO.updateOrder();
			}
			getDaoFactory().commitTransaction();
			result.put("wishLists", wishLists);
			result.put("addresses", addresses);
			result.put("resultCode", "0");

			// If the login was successful, rerun discounts
			if (result.get("resultCode") != null && result.get("resultCode").equals("0")) {
				CartDiscountProcessor discountProcessor = getBusinessObjectFactory()
						.createObject(CartDiscountProcessor.class);
				Map newResults = discountProcessor.processCustomerLogin(null);
				if (newResults != null) {
					result.putAll(newResults);
				}
			}
		}
		return result;
	}

	protected void resetNumberOfFailedLoginsIfLockOutExpired(Customer customer) {
		if (customer != null && customer.getLockedOutUntil() != null && !customer.getLockedOutUntil().equals("")) {
			Date lockedOutUntilDate = parseDateTime(customer.getLockedOutUntil());
			Date now = new Date();
			if (now.after(lockedOutUntilDate)) {
				log.info("Customer " + customer.getUserName()
						+ "'s lock out period expired - resetting number of failed logins.");
				customer.setNumberOfFailedLogins(0);
				customer.setLockedOutUntil(null);
			}
		}
	}

	protected void incrementNumberOfFailedLogins(Customer customer) {
		int numberOfFailedLogins = 0;
		if (customer.getNumberOfFailedLogins() != null) {
			numberOfFailedLogins = customer.getNumberOfFailedLogins();
		}
		customer.setNumberOfFailedLogins(++numberOfFailedLogins);
	}

	protected boolean isCustomerLockedOut(Customer customer) {
		boolean ret = false;
		if (customer != null && customer.getLockedOutUntil() != null && !customer.getLockedOutUntil().equals("")) {
			Date lockedOutUntilDate = parseDateTime(customer.getLockedOutUntil());
			Date now = new Date();
			if (now.before(lockedOutUntilDate)) {
				log.info(
						"Customer " + customer.getUserName() + " is locked out until: " + customer.getLockedOutUntil());
				ret = true;
			}
		}
		return ret;
	}

	protected void lockOutIfTooManyFailedLogins(Customer customer) {
		int numFailedLoginsBeforeLockingOut = 0;
		String numberFailedLoginsBeforeLockingOut = getSettings()
				.getValue("numberFailedLoginsBeforeLockingOutCustomer");
		if (numberFailedLoginsBeforeLockingOut != null && customer.getNumberOfFailedLogins() != null) {
			numFailedLoginsBeforeLockingOut = Integer.parseInt(numberFailedLoginsBeforeLockingOut);
		}
		if (numFailedLoginsBeforeLockingOut > 0
				&& customer.getNumberOfFailedLogins() >= numFailedLoginsBeforeLockingOut) {
			String lockoutDurationInMinutes = getSettings().getValue("lockoutDurationInMinutesCustomer");
			Calendar cal = Calendar.getInstance();
			cal.add(Calendar.MINUTE, Integer.parseInt(lockoutDurationInMinutes));
			String lockedOutUntil = this.formatDateTime(new Date(cal.getTimeInMillis()));
			customer.setLockedOutUntil(lockedOutUntil);
			log.info("Locking out customer " + customer.getUserName() + " due to too many failed logins until: "
					+ customer.getLockedOutUntil());
		}
	}

	protected boolean isPasswordExpired(Customer customer) {
		boolean ret = false;
		if (customer.getPasswordExpires() != null && customer.getPasswordExpires().length() > 0) {
			Date passwordExpiredDate = parseDateTime(customer.getPasswordExpires());
			Date now = new Date();
			if (now.after(passwordExpiredDate)) {
				log.info("Customer password has expired: " + customer.getUserName() + " (expired "
						+ customer.getPasswordExpires() + ").");
				ret = true;
			}
		}
		return ret;
	}

	protected boolean isSameAsPreviousPassword(Customer customer, String newPasswordEncrypted,
			int numberOfPreviousPasswordsToCheck) {
		boolean ret = false;
		if (customer.getPreviousPasswords() != null && numberOfPreviousPasswordsToCheck > 0) {
			String[] previousPasswords = customer.getPreviousPasswords().split(",");
			for (int i = 0; i < previousPasswords.length && i < numberOfPreviousPasswordsToCheck; i++) {
				String previousPassword = previousPasswords[i].trim();
				if (previousPassword.equals(newPasswordEncrypted)) {
					log.info("Customer password same as previous password: " + customer.getUserName());
					ret = true;
				}
			}
		}

		// Also check the current password
		if (numberOfPreviousPasswordsToCheck > 0 && newPasswordEncrypted.equals(customer.getPassword())) {
			log.info("Customer password same as current password: " + customer.getUserName());
			ret = true;
		}
		return ret;
	}

	public Map processChangePassword(String currentDecryptedPassword, String newDecryptedPassword) throws Exception {
		log.info("Changing customer password.");
		Map result = new HashMap();
		CustomerDAO dao = getDaoFactory().createDAO(CustomerDAO.class);
		int numberOfPreviousPasswordsToCheck = Integer
				.parseInt(getSettings().getValue("numberOfPreviousPasswordsToCheckCustomer"));
		String newPasswordEncrypted = dao.getDAOUtils().getEncrypted(newDecryptedPassword,
				(String) getSettings().getValue("customerEncryptionType"),
				getAppSettings().getProperty("twoWayKeyFile"),
				(String) getSettings().getValue("oneWayEncryptionAlgorithm"));
		if (newDecryptedPassword != null && isSameAsPreviousPassword(getUser().getCustomer(), newPasswordEncrypted,
				numberOfPreviousPasswordsToCheck)) {
			result.put("resultCode", "errors.sameAsPreviousPassword");
		} else {
			customerChangePassword(newDecryptedPassword, getUser().getCustomer());
		}
		return result;
	}

	public Map processAutoLogin(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing auto customer login for token: "
					+ parameters.get("autoLoginToken"));
		Map result = new HashMap();
		if (getUser().getCustomer() != null) {
			if (log.isInfoEnabled())
				log.info("Customer is already logged in.");
			result.put("resultCode", "errors.userAlreadyLoggedIn");
			return result;
		}
		getUser().setCustomer(
				(Customer) getBusinessObjectFactory().createObject(
						"customerImplementer"));
		if (parameters != null) {
			BeanUtils.copyProperties(getUser().getCustomer(), parameters);
		}
		CustomerGatewayDAO customerGatewayDAO =  getDaoFactory()
				.createDAO(CustomerGatewayDAO.class);
		result = customerGatewayDAO.processAutoLogin(getUser());
		// If the login was successful, rerun discounts
		if (result.get("resultCode") != null
				&& result.get("resultCode").equals("0")) {
			CartDiscountProcessor discountProcessor =  getBusinessObjectFactory()
					.createObject(CartDiscountProcessor.class);
			Map newResults = discountProcessor.processCustomerLogin(null);
			if (newResults != null) {
				result.putAll(newResults);
			}
		}
		return result;
	}

	public Map updateAddresses(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Updating customer and address.");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException("User is not logged in; cannot update addresses.");
		}
		// Refresh the customer object from the database so admin changes are
		// not overwritten
		CustomerGatewayDAO customerGatewayDAO = getDaoFactory().createDAO(CustomerGatewayDAO.class);
		Map result = customerGatewayDAO.refreshCustomerFromDatabase(getUser());
		if (result.get("resultCode") != null && result.get("resultCode").equals("0")) {
			// Save the current customer in case we have to revert to it
			Map existing = PropertyUtils.describe(getUser().getCustomer());
			// Copy in new parameters
			if (parameters != null) {
				BeanUtils.copyProperties(getUser().getCustomer(), parameters);
			}
			// Create new address object or use existing one.
			CustomerAddress customerAddress = getUser().getCustomer().getFirstAddress();
			if (customerAddress == null) {
				customerAddress = (CustomerAddress) getBusinessObjectFactory().createObject(
						"customerAddressImplementer");
				if (getUser().getCustomer().getCustomerAddressCollection() == null) {
					getUser().getCustomer().setCustomerAddressCollection(new LinkedHashSet());
				}
				getUser().getCustomer().getCustomerAddressCollection().add(customerAddress);
				customerAddress.setCustomer(getUser().getCustomer());
			}
			if (parameters != null) {
				CustomerAddressProcessor cap = getBusinessObjectFactory().createObject(CustomerAddressProcessor.class);
				BeanUtils.copyProperties(customerAddress,
						cap.describeDeliveryOrBillingForAddressBook(parameters, "delivery", true));
			}

			// Update the database
			result = customerGatewayDAO.updateAddresses(getUser().getCustomer());
			// Revert to the previous info if any errors
			if (result != null && result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
				BeanUtils.copyProperties(getUser().getCustomer(), existing);
			}
		}
		return null;
	}

	public Map updateCustomer(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Updating customer");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException("User is not logged in; cannot update customer.");
		}
		// Refresh the customer object from the database so admin changes are
		// not overwritten
		CustomerGatewayDAO customerGatewayDAO = (CustomerGatewayDAO) getDaoFactory().createDAO(
				"customerGatewayDAOImplementer");
		Map result = customerGatewayDAO.refreshCustomerFromDatabase(getUser());
		if (result.get("resultCode") != null && result.get("resultCode").equals("0")) {
		// Save the current customer in case we have to revert to it
		Map existing = PropertyUtils.describe(getUser().getCustomer());
		// Copy in new parameters
		if (parameters != null) {
			BeanUtils.copyProperties(getUser().getCustomer(), parameters);
		}
		// Update the database
			result = customerGatewayDAO.updateCustomer(getUser().getCustomer(), true);
		// Revert to the previous info if any errors
			if (result != null && result.get("resultCode") != null && !result.get("resultCode").equals("0")) {
			BeanUtils.copyProperties(getUser().getCustomer(), existing);
		}
		}
		return result;
	}

	public Map loadCustomerLostPassword(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading customer lost password.");
		Map result = new HashMap();
		// Create a CustomerDAO object to load the customer with
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(getBusinessObjectFactory().createObject(Customer.class));
		// Copy the user name/email address to the object.
		if (parameters != null) {
			BeanUtils.copyProperties(customerDAO.getCustomer(), parameters);
		}
		// Load the customer
		customerDAO.loadFromUserName();
		// Return error if the user name is not found.
		if (customerDAO.getCustomer() == null) {
			result.put("resultCode", "errors.userNameNotFound");
			return result;
		}
		// If the encryption used is one way, reset the password to a random
		// string.
		String customerEncryptionType = (String) getSettings().getValue(
				"customerEncryptionType");
		if (customerEncryptionType.equals("oneWay")) {
			if (log.isInfoEnabled())
				log.info("reseting customer password: "
						+ customerDAO.getCustomer().getUserName());
			Random random = new Random();
			byte[] randomBytes = new byte[8];
			random.nextBytes(randomBytes);
			String randomString = DigestUtils.md5Hex(randomBytes);
			customerDAO.getCustomer().setDecryptedPassword(
					randomString.substring(0, 8));
			customerDAO.updateCustomer(true, false);
		}
		result.put("resultCode", "0");
		result.put("customer", customerDAO.getCustomer());
		return result;
	}

	public Map loadOrderHistory() throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing customer order history.");
		Map result = new HashMap();
		// set the processor's customer property
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot load order history.");
		}
		// Load the order history
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(getUser().getCustomer());
		customerDAO.loadOrderHistory();
		result.put("resultCode", "0");
		return result;
	}

	public Order loadOrderDetails(Map parameters) throws Exception {
		if (log.isInfoEnabled()) {
			log.info("Getting order details for order number: "
					+ parameters.get("orderNumber"));
		}
		// Query for the customer's order
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setCustomerID(new Integer(getUser().getCustomer().getCustomerID()));
		// Copy the order number to the object.
		if (parameters != null) {
			BeanUtils.copyProperties(order, parameters);
		}
		OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
				"orderDAOImplementer");
		orderDAO.setOrder(order);
		// Load the order details
		orderDAO.loadFromNumberAndCustomerID();
		return orderDAO.getOrder();
	}

	public Collection loadDownloadFiles() throws Exception {
		if (log.isInfoEnabled()) {
			log.info("Loading download files for customer.");
		}
		// Make sure customer is logged in
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot load order history.");
		}
		// Load the download files
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(getUser().getCustomer());
		Collection rawDownloads = customerDAO.loadDownloadFiles();
		if (rawDownloads == null || rawDownloads.isEmpty()) {
			return rawDownloads;
		}
		Collection downloads = new LinkedHashSet();
		Iterator i = rawDownloads.iterator();
		int previousProductID = 0;
		Map download = null;
		while (i.hasNext()) {
			Object[] rawDownload = (Object[]) i.next();
			ProductSetting ps = (ProductSetting) rawDownload[0];
			Order o = (Order) rawDownload[1];
			OrderItem oi = (OrderItem) rawDownload[2];

			int productID = ps.getProductID();
			if (productID != previousProductID) {
				if (download != null) {
					downloads.add(download);
				}
				download = new HashMap();
				download.putAll(PropertyUtils.describe(ps));
				download.putAll(PropertyUtils.describe(o));
				download.putAll(PropertyUtils.describe(oi));
			}
			String settingCode = (String) ps.getCode();
			String settingValue = (String) ps.getValue();
			download.put(settingCode, settingValue);
			previousProductID = productID;
		}
		if (download != null) {
			downloads.add(download);
		}
		return downloads;
	}

	public Map saveCart(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Saving cart");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot save cart.");
		}

		// Mark the cart as saved
		getUser().getOrder().setStatus("Saved");

		// Update the database
		OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
				"orderDAOImplementer");
		orderDAO.setOrder(getUser().getOrder());
		orderDAO.updateOrder(true);

		// Return a successful result
		Map result = new HashMap();
		result.put("resultCode", "0");
		return result;
	}

	public Collection loadSavedCarts() throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading customer's saved carts.");
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot load saved carts.");
		}
		// Load the saved carts
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		customerDAO.setCustomer(getUser().getCustomer());
		return customerDAO.loadSavedCarts();
	}

	public Map loadSavedCart(Map parameters) throws Exception {
		if (log.isInfoEnabled()) {
			log.info("Loading saved cart for order ID: "
					+ parameters.get("orderID"));
		}
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot load saved cart.");
		}

		// Query for the customer's order
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setCustomerID(new Integer(getUser().getCustomer().getCustomerID()));
		order.setOrderID(Integer.parseInt((String) parameters.get("orderID")));
		OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
				"orderDAOImplementer");
		orderDAO.setOrder(order);

		// Load the order
		orderDAO.loadSavedCart();

		// Either duplicate the order or reproccess the items, depending on the
		// 'Refresh Carts When Customers Pick Up' setting.
		getUser().setOrder(null);
		OrderProcessor orderProcessor =  getBusinessObjectFactory()
				.createObject(OrderProcessor.class);
		if (getSettings().getValue("refreshSavedCarts").equals("1")) {
			return orderProcessor.reprocessOrder(orderDAO.getOrder());
		} else {
			Order dupeOrder = orderProcessor
					.duplicateOrder(orderDAO.getOrder());
			dupeOrder.setStatus("Copied");
			getUser().setOrder(dupeOrder);
			orderDAO.setOrder(dupeOrder);
			orderDAO.insertOrder(true);
			return null;
		}
	}

	public Map removeSavedCart(Map parameters) throws Exception {
		if (log.isInfoEnabled()) {
			log.info("Removing saved cart for order ID: "
					+ parameters.get("orderID"));
		}
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot load saved cart.");
		}

		// Query for the customer's order
		OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
				"orderDAOImplementer");
		Order order = orderDAO.loadOrderFromID(Integer
				.parseInt((String) parameters.get("orderID")));

		// Check that the order belongs to the customer
		if (order.getCustomerID() == null
				|| order.getCustomerID().intValue() != getUser().getCustomer()
						.getCustomerID()) {
			if (log.isWarnEnabled()) {
				log.warn("Attempt by customerID "
						+ getUser().getCustomer().getCustomerID()
						+ " to remove an order that does not belong to him, orderID: "
						+ parameters.get("orderID"));
			}
			throw new ApplicationException("Wrong order, cannot remove.");
		}

		// Change the status to "Removed"
		order.setStatus("Removed");
		orderDAO.setOrder(order);
		orderDAO.updateOrder(true);
		return null;

	}

	public Map loadPreviousOrder(Map parameters) throws Exception {
		Map results = new HashMap();
		if (log.isInfoEnabled()) {
			log.info("Loading previous order for order ID: "
					+ parameters.get("orderID"));
		}
		if (getUser().getCustomer() == null) {
			throw new ApplicationException(
					"User is not logged in; cannot load previous order.");
		}

		// Query for the customer's order
		Order order = (Order) getBusinessObjectFactory().createObject(
				"orderImplementer");
		order.setCustomerID(new Integer(getUser().getCustomer().getCustomerID()));
		order.setOrderID(Integer.parseInt((String) parameters.get("orderID")));
		OrderDAO orderDAO = (OrderDAO) getDaoFactory().createDAO(
				"orderDAOImplementer");
		orderDAO.setOrder(order);

		// Load the order
		orderDAO.loadPreviousOrder();

		// Either duplicate the order or reproccess the items, depending on the
		// 'Refresh Reorders' setting.
		getUser().setOrder(null);
		OrderProcessor orderProcessor =  getBusinessObjectFactory()
				.createObject(OrderProcessor.class);
		if (getSettings().getValue("refreshReorders").equals("1")) {
			results = orderProcessor.reprocessOrder(orderDAO.getOrder());
			results.put("previousOrder", orderDAO.getOrder());
		} else {
			Order dupeOrder = orderProcessor
					.duplicateOrder(orderDAO.getOrder());
			if (dupeOrder != null) {
				dupeOrder.setStatus("Copied");
				getUser().setOrder(dupeOrder);
				orderDAO.setOrder(dupeOrder);
				orderDAO.insertOrder(true);
			}
			results.put("previousOrder", orderDAO.getOrder());
		}
		return results;
	}

	public Map loadCustomersAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading customers and count");
		CustomerGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CustomerGatewayDAO.class);
		return gatewayDAO.loadCustomersAndCount(parameters);
	}

	public Map addCustomer(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("userName");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding customer: " + code);
		}
		Map results = new HashMap();

		// Check that another customer with the same userName doesn't exist
		Customer customer = (Customer) getBusinessObjectFactory().createObject(
				"customerImplementer");
		BeanUtils.copyProperties(customer, parameters);
		results = validateCustomer(customer);
		if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
			return results;
		}

		customer.setCreated(formatDateTime(new Date()));
		CustomerDAO dao = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		dao.setCustomer(customer);
		dao.insertCustomer(true);
		results.put("newRecord", dao.getCustomer());
		return results;
	}

	public Map validateCustomer(Customer customer) throws Exception {
		Map results = new HashMap();
		CustomerDAO dao = (CustomerDAO) getDaoFactory().createDAO("customerDAOImplementer");
		dao.setCustomer(customer);
		dao.loadFromUserName();
		if (dao.getCustomer() != null) {
			dao.getDaoFactory().getSession().evict(dao.getCustomer());
		}
		boolean insertingButExists = customer.getCustomerID() == 0 && dao.getCustomer() != null
				&& dao.getCustomer().getCustomerID() > 0;
		boolean updatingButExists = customer.getCustomerID() > 0 && dao.getCustomer() != null
				&& customer.getCustomerID() != dao.getCustomer().getCustomerID();
		if (insertingButExists || updatingButExists) {
			results.put("resultCode", "DuplicateRecord");
			results.put("duplicateField", "userName");
			results.put("existingRecord", dao.getCustomer());
		}
		return results;
	}

	public Customer loadCustomerFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading customer from id");
		Customer customer = (Customer) getBusinessObjectFactory().createObject(
				"customerImplementer");
		CustomerDAO dao = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		BeanUtils.copyProperties(customer, parameters);
		dao.setCustomer(customer);
		dao.loadCustomerFromID();
		return dao.getCustomer();
	}

	public Map editCustomer(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("userName");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing customer: " + code);
		}
		Map results = new HashMap();

		// Check that another customer with the same userName doesn't exist
		if (parameters.get("customerID") != null) {
			Customer customer = (Customer) getBusinessObjectFactory().createObject("customerImplementer");
			BeanUtils.copyProperties(customer, parameters);
			results = validateCustomer(customer);
			if (results != null && results.get("resultCode") != null && !results.get("resultCode").equals("0")) {
				return results;
			}
		}

		// Load the customer
		Customer customer = null;
		if (parameters.get("customerID") != null) {
			customer = loadCustomerFromID(parameters);
		}
		if (customer == null && parameters.get("userName") != null) {
			customer = loadCustomerFromUserName(parameters);
		}

		// Update the customer
		BeanUtils.copyProperties(customer, parameters);
		CustomerDAO dao = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		dao.setCustomer(customer);
		dao.updateCustomer(true);
		results.put("updatedRecord", dao.getCustomer());
		return results;
	}

	public Customer loadCustomerFromUserName(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading customer from userName");
		Customer customer = (Customer) getBusinessObjectFactory().createObject("customerImplementer");
		CustomerDAO dao = (CustomerDAO) getDaoFactory().createDAO("customerDAOImplementer");
		BeanUtils.copyProperties(customer, parameters);
		dao.setCustomer(customer);
		dao.loadFromUserName();
		return dao.getCustomer();
	}

	public Map editDeleteCustomers(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Editing/deleting multiple customers");
		CustomerGatewayDAO dao =  getDaoFactory().createDAO(CustomerGatewayDAO.class);
		Map results = dao.editDeleteCustomers(parameters, getBusinessObjectFactory());
		return results;
	}

	public void deleteCustomer(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("userName");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting customer: " + code);
		}
		Customer customer = (Customer) getBusinessObjectFactory().createObject(
				"customerImplementer");
		BeanUtils.copyProperties(customer, parameters);
		CustomerDAO dao = (CustomerDAO) getDaoFactory().createDAO(
				"customerDAOImplementer");
		dao.setCustomer(customer);
		dao.deleteCustomer(true);
		return;
	}

	public int loadMaxCustomerID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading max customer ID.");
		}
		CustomerGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(CustomerGatewayDAO.class);
		return gatewayDAO.loadMaxCustomerID();
	}

	public Map loadCustomerAdvancedLostPassword(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Loading customer advanced lost password.");
		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO("customerDAOImplementer");
		Customer customer = (Customer) getBusinessObjectFactory().createObject("customerImplementer");
		customerDAO.setCustomer(customer);
		// Copy the user name/email address to the object.
		if (parameters != null) {
			BeanUtils.copyProperties(customer, parameters);
		}
		// Load the customer
		customerDAO.loadFromUserName();
		customer = (Customer) customerDAO.getCustomer();
		// Return error if the user name is not found.
		if (customerDAO.getCustomer() == null) {
			result.put("resultCode", "errors.userNameNotFound");
			return result;
		}

		if (log.isInfoEnabled())
			log.info("Updating password token for: " + customer.getUserName());
		String period = (String) getSettings().getValue("tokenExpirationPeriod");
		customer.setPasswordTokenDate(DateUtils.dateTimeAdd(DateUtils.dateTimeGet(Calendar.getInstance()),
				Integer.valueOf(period), 0, 0));
		Random random = new Random();
		byte[] randomBytes = new byte[8];
		random.nextBytes(randomBytes);
		String randomString = DigestUtils.md5Hex(randomBytes);
		customer.setPasswordToken(randomString.substring(0, 8));
		customerDAO.updateCustomer(true, false);
		result.put("resultCode", "0");
		result.put("customer", customerDAO.getCustomer());
		return result;
	}

	public Map loadCustomerFromResetToken(Map parameters) throws Exception {

		if (log.isInfoEnabled())
			log.info("Loading customer from resets token.");

		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO("customerDAOImplementer");
		Customer customer = (Customer) getBusinessObjectFactory().createObject("customerImplementer");
		customerDAO.setCustomer(customer);

		if (parameters != null) {
			BeanUtils.copyProperties(customer, parameters);
		}

		customerDAO.loadCustomerByPasswordToken();
		customer = (Customer) customerDAO.getCustomer();
		if (customer == null) {
			result.put("resultCode", "errors.userNameNotFound");
			return result;
		}
		if (customer.getPasswordTokenDate().compareTo(DateUtils.now()) < 0) {
			result.put("resultCode", "errors.passwordTokenWindowExpired");
			return result;
		}
		result.put("resultCode", "0");
		result.put("customer", customerDAO.getCustomer());

		return result;
	}

	public Map resetPassword(String token, String newPassword) throws Exception {
		if (log.isInfoEnabled())
			log.info("Reseting password from token.");
		Map result = new HashMap();
		CustomerDAO customerDAO = (CustomerDAO) getDaoFactory().createDAO("customerDAOImplementer");
		Customer customer = (Customer) getBusinessObjectFactory().createObject("customerImplementer");
		customer.setPasswordToken(token);
		customerDAO.setCustomer(customer);
		customerDAO.loadCustomerByPasswordToken();
		customer = (Customer) customerDAO.getCustomer();
		int numberOfPreviousPasswordsToCheck = Integer
				.parseInt(getSettings().getValue("numberOfPreviousPasswordsToCheckCustomer"));
		String newPasswordEncrypted = null;
		if (newPassword != null) {
			newPasswordEncrypted = customerDAO.getDAOUtils().getEncrypted(newPassword,
					getSettings().getValue("customerEncryptionType"), getAppSettings().getProperty("twoWayKeyFile"),
					getSettings().getValue("oneWayEncryptionAlgorithm"));
		}
		if (customer == null) {
			result.put("resultCode", "errors.tokenNotFound");
		} else if (customer.getPasswordTokenDate().compareTo(DateUtils.now()) < 0) {
			result.put("resultCode", "errors.tokenExpired");
		} else if (newPassword != null
				&& isSameAsPreviousPassword(customer, newPasswordEncrypted, numberOfPreviousPasswordsToCheck)) {
			result.put("resultCode", "errors.sameAsPreviousPassword");
		} else {
			customerChangePassword(newPassword, customer);
			result.put("resultCode", "0");
			result.put("customer", customer);
			log.info("Customer " + customer.getUserName() + " reset the password.");
		}
		return result;
	}

	public void customerChangePassword(String newDecryptedPassword, Customer customer) throws Exception {
		CustomerDAO dao = getDaoFactory().createDAO(CustomerDAO.class);
		String newPasswordEncrypted = dao.getDAOUtils().getEncrypted(newDecryptedPassword,
				getSettings().getValue("customerEncryptionType"), getAppSettings().getProperty("twoWayKeyFile"),
				getSettings().getValue("oneWayEncryptionAlgorithm"));
		String passwordExpires = computeAndFormatPasswordExpiration();
		String previousPasswords = formatPreviousPasswords(customer, newPasswordEncrypted);
		customer.setDecryptedPassword(newDecryptedPassword);
		customer.setPassword(newPasswordEncrypted);
		customer.setPasswordExpires(passwordExpires);
		customer.setPreviousPasswords(previousPasswords);

		// If the user resets his password successfully, he should be able to use it immediately.
		customer.setNumberOfFailedLogins(0);
		customer.setLockedOutUntil(null);
		dao.setCustomer(customer);
		dao.updateCustomer(true);
	}

	protected String computeAndFormatPasswordExpiration() {
		String passwordExpires = null;
		int passwordExpirationInDays = Integer.parseInt(getSettings().getValue("passwordExpirationInDaysCustomer"));
		if (passwordExpirationInDays > 0) {
			Calendar cal = Calendar.getInstance();
			cal.add(Calendar.DATE, passwordExpirationInDays);
			passwordExpires = this.formatDateTime(new Date(cal.getTimeInMillis()));
		}
		return passwordExpires;
	}

	protected String formatPreviousPasswords(Customer customer, String newPasswordEncrypted) {
		StringBuffer previousPasswords = new StringBuffer("");
		previousPasswords.append(newPasswordEncrypted);
		previousPasswords.append(",");
		if (customer.getPreviousPasswords() != null) {
			String[] prevPasswords = customer.getPreviousPasswords().split(",");
			for (int i = 0; i < prevPasswords.length && i < 20; i++) {
				String previousPassword = prevPasswords[i].trim();
				previousPasswords.append(previousPassword);
				previousPasswords.append(",");
			}
		}
		return previousPasswords.toString();
	}

}
