package com.softslate.commerce.businessobjects.order;

import java.io.Serializable;
import java.text.NumberFormat;
import java.util.Collection;
import java.util.Currency;
import java.util.Locale;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;

/**
 * Object representing an order delivery within the system.
 * <code>OrderDeliveryBean</code> is the default implementation of the
 * <code>OrderDelivery</code> interface for the application.
 * <p>
 * The database stores order deliveries in the <code>sscOrderDelivery</code>
 * table, and this interface can be used to represent a single row of that
 * table. The Hibernate configuration file <code>OrderDelivery.hbm.xml</code>
 * maps that table's columns to the properties in this object.
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.order.OrderDelivery
 */
@Audited
@AuditTable("AudOrderDelivery")
public class OrderDeliveryBean extends BaseBusinessObject implements
		OrderDelivery, Serializable {
	private static final long serialVersionUID = 4050761572458772534L;

	static Log log = LogFactory.getLog(OrderDeliveryBean.class);

	private int orderDeliveryID = 0;

	public int getOrderDeliveryID() {
		return (this.orderDeliveryID);
	}

	public void setOrderDeliveryID(int orderDeliveryID) {
		this.orderDeliveryID = orderDeliveryID;
	}

	private int orderID;

	public int getOrderID() {
		return (this.orderID);
	}

	public void setOrderID(int orderID) {
		this.orderID = orderID;
	}

	private String trackingNumber;

	public String getTrackingNumber() {
		return (this.trackingNumber);
	}

	public void setTrackingNumber(String trackingNumber) {
		this.trackingNumber = trackingNumber;
	}

	private Double total;

	public Double getTotal() {
		return (this.total);
	}

	public void setTotal(Double total) {
		this.total = total;
	}

	private Double shipping;

	public Double getShipping() {
		return (this.shipping);
	}

	public void setShipping(Double shipping) {
		this.shipping = shipping;
	}

	private Double tax;

	public Double getTax() {
		return (this.tax);
	}

	public void setTax(Double tax) {
		this.tax = tax;
	}

	private Double taxableSubtotal;

	public Double getTaxableSubtotal() {
		return (this.taxableSubtotal);
	}

	public void setTaxableSubtotal(Double taxableSubtotal) {
		this.taxableSubtotal = taxableSubtotal;
	}

	private Double subtotal;

	public Double getSubtotal() {
		return (this.subtotal);
	}

	public void setSubtotal(Double subtotal) {
		this.subtotal = subtotal;
	}

	private Double weight;

	public Double getWeight() {
		return (this.weight);
	}

	public void setWeight(Double weight) {
		this.weight = weight;
	}

	private Integer quantity;

	public Integer getQuantity() {
		return (this.quantity);
	}

	public void setQuantity(Integer quantity) {
		this.quantity = quantity;
	}

	private String formattedTotal;

	public String getFormattedTotal() {
		return (this.formattedTotal);
	}

	public void setFormattedTotal(String formattedTotal) {
		this.formattedTotal = formattedTotal;
	}

	private String formattedShipping;

	public String getFormattedShipping() {
		return (this.formattedShipping);
	}

	public void setFormattedShipping(String formattedShipping) {
		this.formattedShipping = formattedShipping;
	}

	private String formattedTax;

	public String getFormattedTax() {
		return (this.formattedTax);
	}

	public void setFormattedTax(String formattedTax) {
		this.formattedTax = formattedTax;
	}

	private String formattedTaxableSubtotal;

	public String getFormattedTaxableSubtotal() {
		return (this.formattedTaxableSubtotal);
	}

	public void setFormattedTaxableSubtotal(String formattedTaxableSubtotal) {
		this.formattedTaxableSubtotal = formattedTaxableSubtotal;
	}

	private String formattedSubtotal;

	public String getFormattedSubtotal() {
		return (this.formattedSubtotal);
	}

	public void setFormattedSubtotal(String formattedSubtotal) {
		this.formattedSubtotal = formattedSubtotal;
	}

	private String lastModified = null;

	public String getLastModified() {
		return (this.lastModified);
	}

	public void setLastModified(String lastModified) {
		this.lastModified = lastModified;
	}

	private String formattedLastModified = null;

	public String getFormattedLastModified() {
		return (this.formattedLastModified);
	}

	public void setFormattedLastModified(String formattedLastModified) {
		this.formattedLastModified = formattedLastModified;
	}

	private String delivered = null;

	public String getDelivered() {
		return (this.delivered);
	}

	public void setDelivered(String delivered) {
		this.delivered = delivered;
	}

	private String formattedDelivered = null;

	public String getFormattedDelivered() {
		return (this.formattedDelivered);
	}

	public void setFormattedDelivered(String formattedDelivered) {
		this.formattedDelivered = formattedDelivered;
	}

	private String status = null;

	public String getStatus() {
		return (this.status);
	}

	public void setStatus(String status) {
		this.status = status;
	}

	private String statusDetails = null;

	public String getStatusDetails() {
		return (this.statusDetails);
	}

	public void setStatusDetails(String statusDetails) {
		this.statusDetails = statusDetails;
	}

	private String firstName = null;

	public String getFirstName() {
		return (this.firstName);
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	private String lastName = null;

	public String getLastName() {
		return (this.lastName);
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	private String organization = null;

	public String getOrganization() {
		return (this.organization);
	}

	public void setOrganization(String organization) {
		this.organization = organization;
	}

	private String address1 = null;

	public String getAddress1() {
		return (this.address1);
	}

	public void setAddress1(String address1) {
		this.address1 = address1;
	}

	private String address2 = null;

	public String getAddress2() {
		return (this.address2);
	}

	public void setAddress2(String address2) {
		this.address2 = address2;
	}

	private String city = null;

	public String getCity() {
		return (this.city);
	}

	public void setCity(String city) {
		this.city = city;
	}

	private String state = null;

	public String getState() {
		return (this.state);
	}

	public void setState(String state) {
		this.state = state;
	}

	private String country = null;

	public String getCountry() {
		return (this.country);
	}

	public void setCountry(String country) {
		this.country = country;
	}

	private String postalCode = null;

	public String getPostalCode() {
		return (this.postalCode);
	}

	public void setPostalCode(String postalCode) {
		this.postalCode = postalCode;
	}

	private String phone1 = null;

	public String getPhone1() {
		return (this.phone1);
	}

	public void setPhone1(String phone1) {
		this.phone1 = phone1;
	}

	private String phone2 = null;

	public String getPhone2() {
		return (this.phone2);
	}

	public void setPhone2(String phone2) {
		this.phone2 = phone2;
	}

	private String email1 = null;

	public String getEmail1() {
		return (this.email1);
	}

	public void setEmail1(String email1) {
		this.email1 = email1;
	}

	private String email2 = null;

	public String getEmail2() {
		return (this.email2);
	}

	public void setEmail2(String email2) {
		this.email2 = email2;
	}

	private String extra1 = null;

	public String getExtra1() {
		return (this.extra1);
	}

	public void setExtra1(String extra1) {
		this.extra1 = extra1;
	}

	private String extra2 = null;

	public String getExtra2() {
		return (this.extra2);
	}

	public void setExtra2(String extra2) {
		this.extra2 = extra2;
	}

	private String extra3 = null;

	public String getExtra3() {
		return (this.extra3);
	}

	public void setExtra3(String extra3) {
		this.extra3 = extra3;
	}

	private String extra4 = null;

	public String getExtra4() {
		return (this.extra4);
	}

	public void setExtra4(String extra4) {
		this.extra4 = extra4;
	}

	private String extra5 = null;

	public String getExtra5() {
		return (this.extra5);
	}

	public void setExtra5(String extra5) {
		this.extra5 = extra5;
	}

	private String notes = null;

	public String getNotes() {
		return (this.notes);
	}

	public void setNotes(String notes) {
		this.notes = notes;
	}

	private Collection orderItemCollection = null;

	public Collection getOrderItemCollection() {
		return (this.orderItemCollection);
	}

	public void setOrderItemCollection(Collection orderItemCollection) {
		this.orderItemCollection = orderItemCollection;
	}

	private Boolean isResidential = null;

	public Boolean getIsResidential() {
		return (this.isResidential);
	}

	public void setIsResidential(Boolean isResidential) {
		this.isResidential = isResidential;
	}

	// Aliases to the above getters and setters...

	public String getDeliveryFirstName() {
		return (getFirstName());
	}

	public void setDeliveryFirstName(String deliveryFirstName) {
		setFirstName(deliveryFirstName);
	}

	public String getDeliveryLastName() {
		return (getLastName());
	}

	public void setDeliveryLastName(String deliveryLastName) {
		setLastName(deliveryLastName);
	}

	public String getDeliveryOrganization() {
		return (getOrganization());
	}

	public void setDeliveryOrganization(String deliveryOrganization) {
		setOrganization(deliveryOrganization);
	}

	public String getDeliveryAddress1() {
		return (getAddress1());
	}

	public void setDeliveryAddress1(String deliveryAddress1) {
		setAddress1(deliveryAddress1);
	}

	public String getDeliveryAddress2() {
		return (getAddress2());
	}

	public void setDeliveryAddress2(String deliveryAddress2) {
		setAddress2(deliveryAddress2);
	}

	public String getDeliveryCity() {
		return (getCity());
	}

	public void setDeliveryCity(String deliveryCity) {
		setCity(deliveryCity);
	}

	public String getDeliveryState() {
		return (getState());
	}

	public void setDeliveryState(String deliveryState) {
		setState(deliveryState);
	}

	public String getDeliveryCountry() {
		return (getCountry());
	}

	public void setDeliveryCountry(String deliveryCountry) {
		setCountry(deliveryCountry);
	}

	public String getDeliveryPostalCode() {
		return (getPostalCode());
	}

	public void setDeliveryPostalCode(String deliveryPostalCode) {
		setPostalCode(deliveryPostalCode);
	}

	public String getDeliveryPhone1() {
		return (getPhone1());
	}

	public void setDeliveryPhone1(String deliveryPhone1) {
		setPhone1(deliveryPhone1);
	}

	public String getDeliveryPhone2() {
		return (getPhone2());
	}

	public void setDeliveryPhone2(String deliveryPhone2) {
		setPhone2(deliveryPhone2);
	}

	public String getDeliveryEmail1() {
		return (getEmail1());
	}

	public void setDeliveryEmail1(String deliveryEmail1) {
		setEmail1(deliveryEmail1);
	}

	public String getDeliveryEmail2() {
		return (getEmail2());
	}

	public void setDeliveryEmail2(String deliveryEmail2) {
		setEmail2(deliveryEmail2);
	}

	public String getDeliveryExtra1() {
		return (getExtra1());
	}

	public void setDeliveryExtra1(String deliveryExtra1) {
		setExtra1(deliveryExtra1);
	}

	public String getDeliveryExtra2() {
		return (getExtra2());
	}

	public void setDeliveryExtra2(String deliveryExtra2) {
		setExtra2(deliveryExtra2);
	}

	public String getDeliveryExtra3() {
		return (getExtra3());
	}

	public void setDeliveryExtra3(String deliveryExtra3) {
		setExtra3(deliveryExtra3);
	}

	public String getDeliveryExtra4() {
		return (getExtra4());
	}

	public void setDeliveryExtra4(String deliveryExtra4) {
		setExtra4(deliveryExtra4);
	}

	public String getDeliveryExtra5() {
		return (getExtra5());
	}

	public void setDeliveryExtra5(String deliveryExtra5) {
		setExtra5(deliveryExtra5);
	}

	public String getDeliveryNotes() {
		return (getNotes());
	}

	public void setDeliveryNotes(String deliveryNotes) {
		setNotes(deliveryNotes);
	}

	public Boolean getDeliveryIsResidential() {
		return (getIsResidential());
	}

	public void setDeliveryIsResidential(Boolean isResidential) {
		setIsResidential(isResidential);
	}

	private Order order = null;

	public Order getOrder() {
		return order;
	}

	public void setOrder(Order order) {
		this.order = order;
	}

	public void addFormatting(Locale locale, String currencyCode) {
		if (currencyCode == null) {
			currencyCode = "USD";
		}
		NumberFormat nf = NumberFormat.getCurrencyInstance(locale);
		Currency currency = Currency.getInstance(currencyCode);
		nf.setCurrency(currency);
		if (getTotal() != null) {
			setFormattedTotal(nf.format(getTotal().doubleValue()));
		}
		if (getSubtotal() != null) {
			setFormattedSubtotal(nf.format(getSubtotal().doubleValue()));
		}
		if (getTax() != null) {
			setFormattedTax(nf.format(getTax().doubleValue()));
		}
		if (getTaxableSubtotal() != null) {
			setFormattedTaxableSubtotal(nf.format(getTaxableSubtotal()
					.doubleValue()));
		}
		if (getShipping() != null) {
			setFormattedShipping(nf.format(getShipping().doubleValue()));
		}
	}
}
