package com.softslate.commerce.businessobjects.payment;

import java.util.HashMap;
import java.util.Map;

import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDelivery;
import com.softslate.commerce.businessobjects.payment.paypal.PayPalConstants;

/**
 * Processes a payment in the store for a given user's cart through PayPal UK
 * Express Checkout. The three API calls required by PayPal Express integration
 * are handled in the {@link #processSetExpressCheckout},
 * {@link #processGetExpressCheckoutDetails}, and
 * {@link #processDoExpressCheckoutPayment} methods.
 * 
 * @author Ashish Jain
 */
public class PayPalUKExpressProcessor extends BasePayPalNVPProcessor implements
		IPayPalExpressPaymentProcessor {

	private String currentTender = null;

	private String currentExpressStep = null;

	public Map process(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing payment.");

		Map results = processDoExpressCheckoutPayment(parameters);

		return results;
	}

	protected Map preparePaypalRequestAcutalPayload(Map params, Map parameters) {
		if (this.currentExpressStep.equals(PayPalConstants.EXPRESS_ACTION_SET))
			this.prepareSetExpressPaypalPayload(params, parameters);
		else if (this.currentExpressStep
				.equals(PayPalConstants.EXPRESS_ACTION_GET))
			this.prepareGetExpressPaypalPayload(params, parameters);
		else if (this.currentExpressStep
				.equals(PayPalConstants.EXPRESS_ACTION_DO))
			this.prepareDoExpressPaypalPayload(params, parameters);

		return params;
	}

	protected void prepareGetExpressPaypalPayload(Map params, Map parameters) {
		params.put(PayPalConstants.ACTION, this.currentExpressStep);
		params.put(PayPalConstants.TOKEN, getUser().getCustomMap().get(
				"paypalToken"));
	}

	protected void prepareDoExpressPaypalPayload(Map params, Map parameters) {
		Order order = getUser().getOrder();
		// String orderTotalString = order.getTotal().toString();
		String currency = (String) getSettings().getValue("currencyCode");
		params.put(PayPalConstants.CURRENCY, currency);
		params.put(PayPalConstants.ACTION, this.currentExpressStep);
		// params.put("amt", orderTotalString);
		params.put(PayPalConstants.TOKEN, getUser().getCustomMap().get(
				"paypalToken"));
		params.put(PayPalConstants.PAYERID, getUser().getCustomMap().get(
				"paypalPayerID"));

		if (order.getSubtotal().doubleValue()
				+ order.getShipping().doubleValue()
				+ order.getTax().doubleValue() == order.getTotal()
				.doubleValue()) {
			// Set the item total (subtotal)
			String itemTotalString = order.getSubtotal().toString();
			params.put(PayPalConstants.ITEM_TOTAL, itemTotalString);

			// Set the shipping total
			String shippingTotalString = order.getShipping().toString();
			params.put(PayPalConstants.FREIGHTAMT, shippingTotalString);

			// Set the tax total
			String taxTotalString = order.getTax().toString();
			params.put(PayPalConstants.TAXAMT, taxTotalString);

			// if (null != order.getOrderNumber())
			// params.put("INVNUM", order.getOrderNumber().toString());
		}

		// Set the order description
		params.put(PayPalConstants.ORDER_DESC, (String) getSettings().getValue(
				"storeName")
				+ "(" + order.getOrderID() + ")");

	}

	protected void prepareSetExpressPaypalPayload(Map params, Map parameters) {

		String orderTotalString = getUser().getOrder().getSubtotal().toString();
		String currency = (String) getSettings().getValue("currencyCode");
		params.put(PayPalConstants.CURRENCY, currency);
		params.put(PayPalConstants.ACTION, this.currentExpressStep);

		String secureURL = (String) getSettings().getValue("customerSecureURL");
		String returnURL = secureURL + "/PayPalExpressReturn.do";
		params.put(PayPalConstants.RETURNURL, returnURL + "?PARTNER="
				+ (String) params.get("PARTNER") + "&amount="
				+ orderTotalString + "&currency=" + currency);

		String customerURL = (String) getSettings().getValue("customerURL");
		String cancelURL = customerURL + "/Cart.do";
		params.put(PayPalConstants.CANCELURL, cancelURL);

		if (null != this.getUser() && null != this.getUser().getCustomer()) {
			String billingEmail1 = this.getUser().getCustomer()
					.getBillingEmail1();
			String email1 = this.getUser().getCustomer().getEmail1();
			String emailId = (null != billingEmail1) ? billingEmail1 : email1;
			if (null != emailId)
				params.put(PayPalConstants.EMAIL, emailId);
		}

	}

	public Map processSetExpressCheckout(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing SetExpressCheckout.");

		if (null == parameters)
			parameters = new HashMap();

		parameters.put(BasePayPalNVPProcessor.CERT_PATH, this.getSettings()
				.getValue("paypalUKCertPath"));

		this.currentTender = PayPalConstants.EXPRESS_PAYMENT_TENDER;
		this.currentExpressStep = PayPalConstants.EXPRESS_ACTION_SET;

		Map result = new HashMap();

		String transactionType = (String) getSettings().getValue(
				"paypalUKPaymentType");
		String trxtype = transactionType;
		if (transactionType.equals(PayPalConstants.AUTH_AUTHENTICATE_TRXTYPE)
				|| transactionType
						.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			trxtype = PayPalConstants.AUTH_TRXTYPE;
		}

		parameters.put(PayPalConstants.TRXTYPE, trxtype);
		parameters.put(PayPalConstants.TENDER, this.currentTender);

		String response = this.makePayPalCall(parameters, "paypalUK");

		// if(stimulate )
		// response = buildSetExpressResponse(true);

		if (log.isDebugEnabled())
			log.debug("response: " + response);

		// Parse the response into a Map
		Map responseMap = parseResponseData(response);

		// Handle the response - errors first
		String resultCode = (String) responseMap.get(PayPalConstants.RESULT);
		if (resultCode == null) {
			result.put("resultCode", "10000");
			return result;
		} else if (!resultCode.equals("0")) {
			String resultMessages[] = new String[1];
			resultMessages[0] = (String) responseMap
					.get(PayPalConstants.RESPMSG);

			result.put("resultCode", resultCode);
			result.put("resultMessages", resultMessages);
			return result;
		}

		if (getUser().getCustomMap() == null)
			getUser().setCustomMap(new HashMap());
		getUser().getCustomMap().put("paypalToken",
				responseMap.get(PayPalConstants.TOKEN));
		result.put("resultCode", "0");
		return result;
	}

	public Map processGetExpressCheckoutDetails(Map parameters)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing GetExpressCheckoutDetails.");

		if (null == parameters)
			parameters = new HashMap();

		parameters.put(BasePayPalNVPProcessor.CERT_PATH, this.getSettings()
				.getValue("paypalUKCertPath"));

		// Check that we have a token
		if (getUser().getCustomMap() == null
				|| getUser().getCustomMap().get("paypalToken") == null) {
			return null;
		}

		this.currentTender = PayPalConstants.EXPRESS_PAYMENT_TENDER;
		this.currentExpressStep = PayPalConstants.EXPRESS_ACTION_GET;

		String transactionType = (String) getSettings().getValue(
				"paypalUKPaymentType");
		String trxtype = transactionType;
		if (transactionType.equals(PayPalConstants.AUTH_AUTHENTICATE_TRXTYPE)
				|| transactionType
						.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			trxtype = PayPalConstants.AUTH_TRXTYPE;
		}

		parameters.put(PayPalConstants.TRXTYPE, trxtype);
		parameters.put(PayPalConstants.TENDER, this.currentTender);

		String response = this.makePayPalCall(parameters, "paypalUK");

		// if(stimulate )
		// response = buildGetExpressResponse(true);

		if (log.isDebugEnabled())
			log.debug("response: " + response);

		// Parse the response into a Map
		Map responseMap = parseResponseData(response);

		// Handle the response - errors first
		String resultCode = (String) responseMap.get(PayPalConstants.RESULT);
		Map result = new HashMap();
		if (resultCode == null) {
			result.put("resultCode", "10000");
			return result;
		} else if (!resultCode.equals("0")) {
			String resultMessages[] = new String[1];
			resultMessages[0] = (String) responseMap
					.get(PayPalConstants.RESPMSG);

			result.put("resultCode", resultCode);
			result.put("resultMessages", resultMessages);
			return result;
		}

		// Store the PayPal shipping details in the user's order
		if (log.isDebugEnabled())
			log
					.debug("Successful paypal response - store info in user's order");

		/*
		 * RESULT=0&RESPMSG=Approved&AVSADDR=Y&TOKEN=EC-17C76533PL706494P&PAYERID=FHY4JXY7CV9PG
		 * &EMAIL=buyer_name@aol.com &PAYERSTATUS=verified &CUSTOM=TRVV14459
		 * &BUSINESS=Monroe Creek Regional Interiors &SHIPTOSTREET=5262 Green
		 * Street #8 &SHIPTOCITY=San Jose &SHIPTOSTATE=CA &SHIPTOZIP=95148
		 * &SHIPTOCOUNTRY=US
		 */

		OrderDelivery fDelivery = getUser().getOrder().firstDelivery();
		if (responseMap.get(PayPalConstants.FIRSTNAME) != null)
			fDelivery.setFirstName((String) responseMap
					.get(PayPalConstants.FIRSTNAME));
		if (responseMap.get(PayPalConstants.LASTNAME) != null)
			fDelivery.setLastName((String) responseMap
					.get(PayPalConstants.LASTNAME));

		if (responseMap.get(PayPalConstants.SHIPTOSTREET) != null)
			fDelivery.setAddress1((String) responseMap
					.get(PayPalConstants.SHIPTOSTREET));

		if (responseMap.get(PayPalConstants.SHIPTOCITY) != null)
			fDelivery.setCity((String) responseMap
					.get(PayPalConstants.SHIPTOCITY));

		if (responseMap.get(PayPalConstants.SHIPTOSTATE) != null)
			fDelivery.setState((String) responseMap
					.get(PayPalConstants.SHIPTOSTATE));

		if (responseMap.get(PayPalConstants.SHIPTOZIP) != null)
			fDelivery.setPostalCode((String) responseMap
					.get(PayPalConstants.SHIPTOZIP));

		if (responseMap.get(PayPalConstants.SHIPTOCOUNTRY) != null)
			fDelivery.setCountry((String) responseMap
					.get(PayPalConstants.SHIPTOCOUNTRY));

		// Populate the billing details
		getUser().getOrder().setAddress1("[PayPal]");
		getUser().getOrder().setAddress2("[PayPal]");
		getUser().getOrder().setCity("[PayPal]");
		getUser().getOrder().setState("[PayPal]");
		getUser().getOrder().setPostalCode("[PayPal]");
		getUser().getOrder().setFirstName(
				(String) responseMap.get(PayPalConstants.FIRSTNAME));

		getUser().getOrder().setLastName(
				(String) responseMap.get(PayPalConstants.LASTNAME));

		getUser().getOrder().setOrganization(
				(String) responseMap.get(PayPalConstants.BUSINESS));

		getUser().getOrder().setCountry(
				(String) responseMap.get(PayPalConstants.SHIPTOCOUNTRY));

		getUser().getOrder().setPhone1(
				(String) responseMap.get(PayPalConstants.PHONENUM));
		getUser().getOrder().setEmail1(
				(String) responseMap.get(PayPalConstants.EMAIL));

		// Store other custom things
		getUser().getCustomMap().put("paypalPayerID",
				responseMap.get(PayPalConstants.PAYERID));
		getUser().getCustomMap().put("paypalPayerStatus",
				responseMap.get(PayPalConstants.PAYERSTATUS));

		result.put("resultCode", "0");
		return result;
	}

	public Map processDoExpressCheckoutPayment(Map parameters) throws Exception {
		if (log.isInfoEnabled())
			log.info("Processing DoExpressCheckoutPayment.");

		if (null == parameters)
			parameters = new HashMap();

		parameters.put(BasePayPalNVPProcessor.CERT_PATH, this.getSettings()
				.getValue("paypalUKCertPath"));

		// Check that we have a token and a Payer ID
		if (getUser().getCustomMap() == null
				|| getUser().getCustomMap().get("paypalToken") == null
				|| getUser().getCustomMap().get("paypalPayerID") == null) {
			if (log.isInfoEnabled())
				log.info("No token or payer ID for user.");
			return null;
		}

		this.currentTender = PayPalConstants.EXPRESS_PAYMENT_TENDER;
		this.currentExpressStep = PayPalConstants.EXPRESS_ACTION_DO;

		Map result = new HashMap();

		String transactionType = (String) getSettings().getValue(
				"paypalUKPaymentType");
		String trxtype = transactionType;
		if (transactionType.equals(PayPalConstants.AUTH_AUTHENTICATE_TRXTYPE)
				|| transactionType
						.equals(PayPalConstants.AUTH_AUTHENTICATE_SALE_TRXTYPE)) {
			trxtype = PayPalConstants.AUTH_TRXTYPE;
		}

		parameters.put(PayPalConstants.TRXTYPE, trxtype);
		parameters.put(PayPalConstants.TENDER, this.currentTender);

		String response = this.makePayPalCall(parameters, "paypalUK");

		if (log.isDebugEnabled())
			log.debug("response: " + response);

		// Parse the response into a Map
		Map responseMap = parseResponseData(response);

		// Handle the response - errors first
		String resultCode = (String) responseMap.get(PayPalConstants.RESULT);
		result.put("resultCode", resultCode);
		if (resultCode == null) {
			result.put("resultCode", "10000");
			return result;
		} else if (!resultCode.equals("0")) {
			String resultMessages[] = new String[1];
			resultMessages[0] = (String) responseMap
					.get(PayPalConstants.RESPMSG);
			result.put("resultMessages", resultMessages);
			return result;
		}

		processSuccessfulTransaction(parameters, responseMap);

		return result;
	}

	protected void setPaymentVariablesToPersist(Payment payment,
			Map parameters, Map responseMap) {
		payment.setOrder(getUser().getOrder());
		payment
				.setValueFieldLabels("Order Total,PayPal Token,Payment Status,Message,Payment Type,Transaction Type,Transaction ID,,");
		payment.setValue1(getUser().getOrder().getTotal().toString());
		payment.setValue2((String) responseMap.get(PayPalConstants.TOKEN));
		payment.setValue3("PENDING_REASON : "
				+ responseMap.get(PayPalConstants.PENDINGREASON));
		payment.setValue4("User elected to pay with PayPal");
		payment
				.setValue5((String) responseMap
						.get(PayPalConstants.PAYMENTTYPE));
		payment.setValue6((String) this.currentExpressStep);
		payment.setValue7((String) responseMap.get(PayPalConstants.PNREF));
	}

}
