package com.softslate.commerce.businessobjects.product;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.softslate.commerce.businessobjects.core.BaseBusinessProcessor;
import com.softslate.commerce.daos.product.OptionDAO;
import com.softslate.commerce.daos.product.OptionGatewayDAO;
import com.softslate.commerce.daos.product.SKUGatewayDAO;

/**
 * Processes business logic having to do with options. Created in the Struts
 * layer in response to requests for viewing, editing, adding, and deleting
 * options.
 * <p>
 * <code>BasicOptionProcessor</code> is the default implementation of the
 * <code>OptionProcessor</code> interface for the application.
 * 
 * @author David Tobey
 * 
 */
public class BasicOptionProcessor extends BaseBusinessProcessor implements
		OptionProcessor {

	static Log log = LogFactory.getLog(BasicOptionProcessor.class);

	public Map loadOptionsAndCount(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading options and count");
		OptionGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(OptionGatewayDAO.class);
		return gatewayDAO.loadOptionsAndCount(parameters);
	}

	public Map addOption(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("optionCode");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Adding option: " + code);
		}
		Map results = new HashMap();

		// Check that another option with the same code doesn't exist
		Option option = (Option) getBusinessObjectFactory().createObject(
				"optionImplementer");
		BeanUtils.copyProperties(option, parameters);
		results = validateOption(option);
		if (results != null && results.get("resultCode") != null
				&& !results.get("resultCode").equals("0")) {
			return results;
		}

		// Insert the option
		Attribute attribute =  getBusinessObjectFactory()
				.createObject(Attribute.class);
		attribute.setAttributeID(option.getAttributeID());
		option.setAttribute(attribute);
		OptionDAO dao = (OptionDAO) getDaoFactory().createDAO(
				"optionDAOImplementer");
		dao.setOption(option);
		dao.insertOption(true);
		results.put("newRecord", dao.getOption());
		return results;
	}

	public Option loadOptionFromID(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading option from id");
		Option option = (Option) getBusinessObjectFactory().createObject(
				"optionImplementer");
		OptionDAO dao = (OptionDAO) getDaoFactory().createDAO(
				"optionDAOImplementer");
		BeanUtils.copyProperties(option, parameters);
		dao.setOption(option);
		dao.loadOptionFromID();
		return dao.getOption();
	}

	public Option loadOptionFromCode(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName() + ": Loading option from code");
		Option option = (Option) getBusinessObjectFactory().createObject("optionImplementer");
		OptionDAO dao = (OptionDAO) getDaoFactory().createDAO("optionDAOImplementer");
		BeanUtils.copyProperties(option, parameters);
		dao.setOption(option);
		dao.loadOptionFromCode();
		return dao.getOption();
	}

	public Map editOption(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("optionCode");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing option: " + code);
		}
		Map results = new HashMap();

		// Check that another option with the same code doesn't exist
		Option option = (Option) getBusinessObjectFactory().createObject(
				"optionImplementer");
		BeanUtils.copyProperties(option, parameters);
		results = validateOption(option);
		if (results != null && results.get("resultCode") != null
				&& !results.get("resultCode").equals("0")) {
			return results;
		}

		// Load the option
		option = loadOptionFromID(parameters);
		BeanUtils.copyProperties(option, parameters);

		// Update the option
		Attribute attribute =  getBusinessObjectFactory()
				.createObject(Attribute.class);
		attribute.setAttributeID(option.getAttributeID());
		option.setAttribute(attribute);
		OptionDAO dao = (OptionDAO) getDaoFactory().createDAO(
				"optionDAOImplementer");
		dao.setOption(option);
		dao.updateOption(true);
		results.put("updatedRecord", dao.getOption());
		return results;
	}

	public Map editDeleteOptions(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null)
			log.info(getUser().getAdministrator().getUserName()
					+ ": Editing/deleting multiple options");
		OptionGatewayDAO dao = (OptionGatewayDAO) getDaoFactory().createDAO(
				"optionGatewayDAOImplementer");
		dao.editDeleteOptions(parameters, getBusinessObjectFactory());
		SKUGatewayDAO skuDao = (SKUGatewayDAO) getDaoFactory().createDAO(
				"skuGatewayDAOImplementer");
		skuDao.deleteOrphanedSKUs(true);
		return null;
	}

	public Map deleteOption(Map parameters) throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			String code = (String) parameters.get("optionCode");
			log.info(getUser().getAdministrator().getUserName()
					+ ": Deleting option: " + code);
		}
		Option option = (Option) getBusinessObjectFactory().createObject(
				"optionImplementer");
		BeanUtils.copyProperties(option, parameters);
		OptionDAO dao = (OptionDAO) getDaoFactory().createDAO(
				"optionDAOImplementer");
		dao.setOption(option);
		dao.deleteOption(false);
		SKUGatewayDAO skuDao = (SKUGatewayDAO) getDaoFactory().createDAO(
				"skuGatewayDAOImplementer");
		skuDao.deleteOrphanedSKUs(true);
		return null;
	}

	public Map validateOption(Option option) throws Exception {
		Map results = new HashMap();
		OptionDAO dao = (OptionDAO) getDaoFactory().createDAO(
				"optionDAOImplementer");
		dao.setOption(option);
		dao.loadOptionFromCode();
		if (dao.getOption() != null) {
			dao.getDaoFactory().getSession().evict(dao.getOption());
		}
		boolean insertingButExists = option.getOptionID() == 0
				&& dao.getOption() != null && dao.getOption().getOptionID() > 0;
		boolean updatingButExists = option.getOptionID() > 0
				&& dao.getOption() != null
				&& option.getOptionID() != dao.getOption().getOptionID();
		if (insertingButExists || updatingButExists) {
			results.put("resultCode", "DuplicateRecord");
			results.put("duplicateField", "optionCode");
			results.put("existingRecord", dao.getOption());
		}
		return results;
	}

	public int loadMaxOptionID() throws Exception {
		if (log.isInfoEnabled() && getUser().getAdministrator() != null) {
			log.info(getUser().getAdministrator().getUserName()
					+ ": Loading max option ID.");
		}
		OptionGatewayDAO gatewayDAO =  getDaoFactory()
				.createDAO(OptionGatewayDAO.class);
		return gatewayDAO.loadMaxOptionID();
	}
}
