package com.softslate.commerce.businessobjects.shipping;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface for objects that are able to process shipping charges in the store.
 * Custom shipping processors must simply implement
 * <code>ShippingProcessor</code>.
 * <p>
 * To change or add to the list of shipping processors currently active in the
 * system, modify the "activeShippingProcessors" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * During checkout, the Struts layer invokes each of the
 * "activeShippingProcessors" by calling their
 * <code>loadShippingOptions()</code> method to get a list of options related
 * to this <code>ShippingProcessor</code>, and
 * <code>processShipping(Map parameters)</code> to process the option that was
 * selected. The incoming <code>Map</code> contains the
 * <code>shippingOption</code> selected by the user, along with any other
 * request parameters.
 * <p>
 * The default "activeShippingProcessor" is
 * {@link com.softslate.commerce.businessobjects.shipping.BasicShippingProcessor}.
 * 
 * @author David Tobey
 * 
 */
public interface ShippingProcessor extends BusinessObject {

	/**
	 * Load the shipping options related to this <code>ShippingProcessor</code>.
	 * <p>
	 * Shipping options might include, for example, ground, 2-day air, bulk,
	 * etc.
	 * <p>
	 * The <code>Collection</code> returned should be a collection of
	 * <code>Map</code> objects that must include the following keys:
	 * <ul>
	 * <li>code: A unique <code>String</code> code for the shipping option
	 * under this shipping processor. E.g., "GROUND".</li>
	 * <li>name: The <code>String</code> name of the shipping option for
	 * diplay to users. E.g., "Ground".</li>
	 * <li>price: The formatted price for the shipping option represented as a
	 * <code>String</code> and including currency formatting, for display to
	 * the user.</li>
	 * <li>priceDouble: The raw price for the shipping option represented as a
	 * <code>Double</code>.</li>
	 * </ul>
	 * 
	 * @param parameters
	 *            A Map containing information from the current request.
	 * @return A <code>Collection</code> of <code>Map</code> objects that
	 *         contain the code, name, and price for each shipping option
	 *         related to this <code>ShippingProcessor</code>.
	 * @throws Exception
	 */
	abstract Collection loadShippingOptions(Map parameters) throws Exception;

	/**
	 * Process the shipping charges for the current user's order. The incoming
	 * <code>Map</code> should contain a key named <code>shippingOption</code>
	 * that contains the <code>code</code> of the shipping option selected by
	 * the user. It may also contain additional input parameters related to how
	 * the shipping should be charged.
	 * 
	 * @param parameters
	 *            A Map containing the shipping infofmation submitted by the
	 *            user. It should contain a key named
	 *            <code>shippingOption</code> that contains the
	 *            <code>code</code> of the shipping option selected by the
	 *            user.
	 * @return A Map containing a least one key named "resultCode", which
	 *         contains a String parsable into an integer, representing the
	 *         results of the processing. 0 for the resultCode indicates
	 *         success, anything else indicates an error and that processing
	 *         should stop.
	 * @throws Exception
	 */
	abstract Map processShipping(Map parameters) throws Exception;
}
